@@ -12,6 +12,7 @@ from simtk import openmm as mm
 from simtk.openmm import app
 
 from .custom_reporters import FESReporter
+from .build_CVs import build_CVs
 from .utils import *
 
 
@@ -122,6 +123,7 @@ class MDSimulation():
     traj_out : string, optional
         name of file for writing trajectory information. Defaults to 'trajectory.h5'
     """
+
     def __init__(self, parmed_structure, coordinates=None,
                  velocities=None, box_vectors=None,
                  cwd=None, centering=False,
@@ -131,6 +133,7 @@ class MDSimulation():
                  atoms_to_freeze=None, atoms_to_restrain=None,
                  use_COM_restraint=False,
                  restraint_weight=None, constraints='HBonds',
+                 wrap_protein_ligand=False,
                  state_out='sim.log',
                  implicit_solvent_model=None, hmr=False,
                  state_interval=None, progress_interval=None, traj_interval=None,
@@ -187,6 +190,7 @@ class MDSimulation():
         self.state_out = state_out
         self.atoms_to_restrain = atoms_to_restrain
         self.restraint_weight = restraint_weight
+        self.wrap_protein_ligand = wrap_protein_ligand
         self.implicit_solvent_model = implicit_solvent_model
         self.hmr = hmr
         self.run_time = run_time
@@ -369,6 +373,28 @@ class MDSimulation():
         print('unfroze {} atoms'.format(len(self.frozen_atoms)))
         self.frozen_atoms = {}
 
+    def add_virtual_bond(self, atom_selection1=None, atom_selection2=None):
+        """ Adds a virtual (zero-energy) bond between two sets of atoms, forcing them to be kept in the
+            same box for imaging purposes
+
+            If no atom selections are given selection1 will be ligand and selection2 will be protein
+
+        :param atom_selection1: list of atom indeces
+        :param atom_selection2: list of atom indeces
+        :return:
+        """
+
+        if atom_selection1==None:
+            atom_selection1 = select_atoms(self.parmed_structure,keyword_selection='ligand')
+        if atom_selection2==None:
+            atom_selection2 = select_atoms(self.parmed_structure,keyword_selection='protein')
+
+        virt_bond = mm.CustomCentroidBondForce(2, ' 0 * distance(g1,g2)')
+        virt_bond.addGroup(atom_selection1)
+        virt_bond.addGroup(atom_selection2)
+        virt_bond.addBond([0,1])
+        self.custom_forces['Virtual_Bond'] = self.system.addForce((virt_bond))
+
     def restrain_COM(self, atom_selection, COM_restraint_weight=20):
         """ Adds a positional restraint to the COM of the atom_selection
 
@@ -510,6 +536,10 @@ class MDSimulation():
         if self.atoms_to_freeze is not None:
             self.freeze_atom_selection(self.atoms_to_freeze)
 
+        if self.wrap_protein_ligand:
+            #if no atom selection is given this defaults to protein ligand
+            self.add_virtual_bond()
+
         # get integrator
         if self.integrator_type not in ['Langevin', 'Verlet']:
             raise ValueError('{} integrator type not supported'.format(self.integrator))
@@ -631,6 +661,9 @@ class MDSimulation():
         if not isinstance(self.progress_out,str):
             self.progress_out = "stdout"
 
+        if not isinstance(self.state_out,str):
+            self.state_out = "stdout"
+
         fname = os.path.join(self.cwd, pickle_name)
         with open(fname, 'wb') as handle:
             pickle.dump(obj=self, file=handle)
@@ -973,14 +1006,26 @@ class MetadynamicSimulation(MDSimulation):
     Metadynamics Simulation Class
 
     A maximum of 3 CVs can be specified, and each CV must be one of the following:
-    'Distance, RMSD, Z-Proj, Angle'. Each CV must have exactly one value specified in the following format:
+    'Distance, RMSD, Z-Proj, Angle, Coordination Number and Weighted Distances'.
+    Each CV must have exactly one value specified in the following format:
     [CV_name (string), mask_1 (string), mask_2 (string), CV_min (float), CV_max (float), CV_bin_length (float)]
 
     Masks follow ommtk select atoms syntax where masks beginning with 'resid' are assumed to be resid_selection
     keywords and masks without 'resid' are assumed to be keyword_selection inputs. Where only one
     mask is needed (i.e. for RMSD or Angle) the second mask is ignored, and should be specified as 'none'. CV dimensions
-    should be constructed such that all CVs have the same discrete number of bins, even if the units on those bins
-    are not equal.
+    should be constructed such that all CVs have the same discrete number of bins, even if the phase space covered
+    by those bins are not equal.
+
+    Mask selection now supports lists of atom indeces. For Coordination number and weighted distances each index
+    is assumed to correspond to exactly one donor or receptor atom. The distances are applied in order for the loops.
+    So if you want to bias the distance between atom 100 and atom 200, the distance between atom 110 and 210
+    you can use the following:
+
+    >>> CV_list = [
+    >>>    ['Coordination Number',[100,110],[200,210],.5,1,.025,[1/2,1/2],[.5,.5]]
+    >>>]
+
+    where cv[6] and cv[7] correspond to the weights and the unbound_distance/2 for the atom pairs in cv[1] and cv[2].
 
     All structures should be aligned such that the Distance CV is aligned along the Z-axis. This can be done using
     ommtk align_structure() command, but should be done before solvating the system as rotating a system after
@@ -1017,6 +1062,10 @@ class MetadynamicSimulation(MDSimulation):
     fb_scale_factor : float
         fb restraint radius will automatically be set to the max distance on protein from ligand * this factor
 
+    unbound_distance : float
+        distance between ligand and protein atoms considered to be fully unbound. Used for coordination number and
+        segment mapping sims
+
     radius_scale_factor : float, optional
         factor for determining the radius of funnel or dome relative to ligand size
 
@@ -1051,7 +1100,7 @@ class MetadynamicSimulation(MDSimulation):
                  bias_save_frequency=1 * unit.picoseconds,
                  bias_directory=None,
                  fes_interval=.5 * unit.nanoseconds,
-                 fb_scale_factor=.8,
+                 dome_radius=4, unbound_distance=4,
                  **kwargs):
 
         super().__init__(**kwargs)
@@ -1069,7 +1118,8 @@ class MetadynamicSimulation(MDSimulation):
         self.radius_scale_factor = radius_scale_factor
         self.funnel_angle = funnel_angle
         self.anchor_selection = anchor_selection
-        self.fb_scale_factor = fb_scale_factor
+        self.dome_radius = dome_radius
+        self.unbound_distance = unbound_distance
 
         self._build_custom_forces()
 
@@ -1093,21 +1143,11 @@ class MetadynamicSimulation(MDSimulation):
 
             # Now get the setup stage for placing sphere (otherwise coords may be wrapped)
             lig_atom_index = select_atoms(self.parmed_structure, keyword_selection='ligand')
-            prot_atom_index = select_atoms(self.parmed_structure, keyword_selection='protein')
 
-            lig_positions = np.array(self.positions.value_in_unit(unit.nanometer))[lig_atom_index]
-            prot_positions = np.array(self.positions.value_in_unit(unit.nanometer))[prot_atom_index]
-
-            if len(lig_atom_index) == 0 or len(prot_atom_index) == 0:
-                raise ValueError('could not select ligand or protein for flat bottom restraint')
-
-            #use this function to estimate the radius of the flat bottom pot
-            radius = find_flat_bottom_radius(lig_positions, prot_positions, self.fb_scale_factor)
-
-            print('Adding a flat-bottom restraint with radius: {} '.format(radius))
+            print('Adding a flat-bottom restraint with radius: {} '.format(self.dome_radius))
 
             fb_force = mm.CustomCentroidBondForce(2, 'step(r-r0) * (fb_k) * (r-r0)^2; r=distance(g1,g2)')
-            fb_force.addGlobalParameter('r0',radius)
+            fb_force.addGlobalParameter('r0',self.dome_radius)
             fb_force.addGlobalParameter('fb_k',20)
             # find anchor group
             if self.anchor_selection.startswith('resid'):
@@ -1175,129 +1215,7 @@ class MetadynamicSimulation(MDSimulation):
             for i in lig_atom_index:
                 funnel_force.addParticle(i, [])
 
-        CVs = []
-        self.bin_lengths = []
-        # cv list format will be:
-        #       cv[0] = cv name
-        #       cv[1] = atom_selection1 (should be ligand if using both)
-        #       cv[2] = atom_selection2 (should be protein if possible)
-        #       cv[3] = cv min value
-        #       cv[4] = cv max value
-        #       cv[5] = cv bin width
-
-        print('Building collective variables with the following specifications: {}'.format(self.CV_list))
-
-        for cv in self.CV_list:
-            if cv[0] == 'RMSD':
-
-                if cv[1].startswith('resid'):
-                    resid_list = cv[1][6:]
-                    atom_set = select_atoms(self.parmed_structure, resid_selection=resid_list)
-                else:
-                    atom_set = select_atoms(self.parmed_structure, keyword_selection=cv[1])
-
-                if len(atom_set) == 0:
-                    raise ValueError('RMSD atom selection found no atoms')
-                print('Number of atoms to be used for RMSD Bias: {}'.format(len(atom_set)))
-                rmsd_force = mm.RMSDForce(self.positions, atom_set)
-                rmsd_bias = mm.app.BiasVariable(rmsd_force,
-                                                minValue=cv[3],
-                                                maxValue=cv[4],
-                                                biasWidth=cv[5])
-                self.bin_lengths.append((cv[4] - cv[3]) / cv[5])
-                CVs.append(rmsd_bias)
-
-            if cv[0] == 'Distance':
-                com_force = mm.CustomCentroidBondForce(2, 'distance(g1,g2)')
-
-                # find first group
-                if cv[1].startswith('resid'):
-                    resid_list = cv[1][6:]
-                    atom_group1 = select_atoms(self.parmed_structure, resid_selection=resid_list)
-                else:
-                    atom_group1 = select_atoms(self.parmed_structure, keyword_selection=cv[1])
-
-                # find second group
-                if cv[2].startswith('resid'):
-                    resid_list = cv[2][6:]
-                    atom_group2 = select_atoms(self.parmed_structure, resid_selection=resid_list)
-                else:
-                    atom_group2 = select_atoms(self.parmed_structure, keyword_selection=cv[2])
-
-                if len(atom_group1) == 0 or len(atom_group2) == 0:
-                    raise ValueError('One of the atom selection sets has no atoms')
-
-                print('num atoms being used in 1st moeity: {}'.format(len(atom_group1)))
-                print('num atoms being used in 2nd moeity: {}'.format(len(atom_group2)))
-                com_force.addGroup(atom_group1)
-                com_force.addGroup(atom_group2)
-                com_force.addBond([0, 1])
-                com_bias = mm.app.BiasVariable(com_force,
-                                               minValue=cv[3],
-                                               maxValue=cv[4],
-                                               biasWidth=cv[5])
-                CVs.append(com_bias)
-
-                self.bin_lengths.append((cv[4] - cv[3]) / cv[5])
-
-            if cv[0] == 'Z-Proj':
-                # z-proj is just a distance force using only the z components
-                zproj_force = mm.CustomCentroidBondForce(2, 'abs(z1-z2)')
-
-                # find first group
-                if cv[1].startswith('resid'):
-                    resid_list = cv[1][6:]
-                    atom_group1 = select_atoms(self.parmed_structure, resid_selection=resid_list)
-                else:
-                    atom_group1 = select_atoms(self.parmed_structure, keyword_selection=cv[1])
-
-                # find second group
-                if cv[2].startswith('resid'):
-                    resid_list = cv[2][6:]
-                    atom_group2 = select_atoms(self.parmed_structure, resid_selection=resid_list)
-                else:
-                    atom_group2 = select_atoms(self.parmed_structure, keyword_selection=cv[2])
-
-                if len(atom_group1) == 0 or len(atom_group2) == 0:
-                    raise ValueError('One of the atom selection sets has no atoms')
-
-                zproj_force.addGroup(atom_group1)
-                zproj_force.addGroup(atom_group2)
-                zproj_force.addBond([0, 1])
-                z_bias = mm.app.BiasVariable(zproj_force,
-                                             minValue=cv[3],
-                                             maxValue=cv[4],
-                                             biasWidth=cv[5])
-                self.bin_lengths.append((cv[4] - cv[3]) / cv[5])
-                CVs.append(z_bias)
-
-            if cv[0] == 'Angle':
-                # angle force can only be applied to three atoms, but they don't need to be bound
-                # if we want to support angles between groups we can use CustomCentroidBondForce
-                angle_force = mm.CustomAngleForce('theta')
-
-                lig_atom_index = select_atoms(self.parmed_structure, keyword_selection='ligand')
-                prot_atom_index = select_atoms(self.parmed_structure, keyword_selection='protein')
-                lig_positions = np.array(self.positions.value_in_unit(unit.nanometer))[lig_atom_index]
-                prot_positions = np.array(self.positions.value_in_unit(unit.nanometer))[prot_atom_index]
-                three_coords, indeces = get_angle_particle_coords(lig_positions, prot_positions)
-                print('Using atoms {}, {}, {} for angle bias'.format(indeces[0], indeces[1], indeces[2]))
-                angle_force.addAngle(indeces[0], indeces[1], indeces[2])
-                angle_bias = mm.app.BiasVariable(angle_force,
-                                                 minValue=cv[3],
-                                                 maxValue=cv[4],
-                                                 biasWidth=cv[5],
-                                                 periodic=True)
-                self.bin_lengths.append((cv[4] - cv[3]) / cv[5])
-                CVs.append(angle_bias)
-
-
-        #all bins must be the same shape
-        if not self.bin_lengths[1:] == self.bin_lengths[:-1]:
-            raise ValueError(
-                'Bias dimensions must be equal for all CVs specified. Please adjust the max, min, and binwidths',
-                'such that max-min/width are equal for all CVs')
-
+        CVs = build_CVs(self.CV_list,self.parmed_structure,self.system)
 
         # build sim
         self.metaD = mm.app.Metadynamics(system=self.system,
