@@ -1,5 +1,6 @@
 
 import numpy as np
+from scipy import signal
 from simtk.openmm import unit
 
 
@@ -69,7 +70,9 @@ def select_atoms(parmed_structure, keyword_selection=None, ligand_resname=None,
             # convert letter rep of chain to ordinal number rep
             if ord(chain_selection.lower()) - 96 == k + 1:
                 residues = [i for i in chain.residues()]
-                selected_residues = [i for i in residues if i.index in resid_list]
+                res0 = residues[0]
+                start_index = res0.index
+                selected_residues = [i for i in residues if i.index - start_index in resid_list]
 
                 if len(selected_residues) == 0:
                     raise ValueError('Could not find one of the residues {} on protein chain.'.format(resid_list))
@@ -307,6 +310,27 @@ def find_contacts(positions1, positions2, cutoff):
 
     return contacts_i,contacts_j
 
+
+def find_com_distance_between_atom_groups(parmed, masses, group1, group2):
+    """
+    given two groups of atom indeces and a parmed.structure object and the masses,
+    return the distance between the center of masses of the atom groups
+    :param parmed: Parmed.structure.Structure()
+    :param masses: list of integers or floats
+    :param group1: list of integers
+    :param group2: list integers
+    :return: float
+    """
+    g1_coords = np.array(parmed.positions.value_in_unit(unit.nanometer))[group1]
+    g2_coords = np.array(parmed.positions.value_in_unit(unit.nanometer))[group2]
+    g1_masses = np.array(masses)[group1]
+    g2_masses = np.array(masses)[group2]
+    g1_com = find_center_of_mass(g1_coords, g1_masses)
+    g2_com = find_center_of_mass(g2_coords, g2_masses)
+    distance = dist(g1_com, g2_com)
+    return distance
+
+
 def get_bound_site_cog(parmed, cutoff_in_angstroms):
     """
 
@@ -434,3 +458,50 @@ def align_structure(pmd, sel1, sel2, axis):
     pmd.positions = rotated_coordinates * unit.angstrom
 
     return pmd
+
+def calc_dG(fes,well_combining_method):
+    """
+    Input a free energy surface and calculate energy difference between wells. It is agnostic to units
+
+    :param fes: 2D matrix in format given by FESResporter
+    :param well_combining_method: string ['mean','max']
+    :return: dG estimate (float)
+    """
+    fes = np.array(fes)
+    minimax = signal.argrelextrema(fes, np.less)
+    minimay = signal.argrelextrema(fes, np.less, axis=1)
+
+    y_direction = set()
+    for i in range(len(minimay[0])):
+        y_direction.add((minimay[0][i], minimay[1][i]))
+    x_direction = set()
+    for i in range(len(minimax[0])):
+        x_direction.add((minimax[0][i], minimax[1][i]))
+
+    min_set = x_direction.intersection(y_direction)
+    min_set = list(min_set)
+    x_mins = [min_set[i][0] for i in range(len(min_set))]
+    y_mins = [min_set[i][1] for i in range(len(min_set))]
+
+    dG_list = []
+    try:
+        min_index = np.argmin([fes[x_mins[i], y_mins[i]] for i in range(len(x_mins))])
+    except ValueError:
+        print('did not find any wells')
+        return 0
+
+    for i in range(len(x_mins)):
+        if i != min_index and y_mins[i] - y_mins[min_index] > 10:
+            dG_val = fes[x_mins[min_index], y_mins[min_index]] - fes[x_mins[i], y_mins[i]]
+            dG_list.append(dG_val)
+
+    if len(dG_list) == 0:
+        print('dG_list empty')
+        dG = 0
+    else:
+        if well_combining_method=='mean':
+            dG = np.mean(dG_list)
+        if well_combining_method=='max':
+            dG = np.max(dG_list)
+
+    return dG
